"""Scanner module that run scanner command in daemon mode."""
import asyncio
import logging
import sys
import socket
import multiprocessing
from typing import Optional

import click

from ostorlab import configuration_manager as config_manager
from ostorlab.cli import console as cli_console
from ostorlab.cli.rootcli import rootcli
from ostorlab.scanner import scan_handler
from ostorlab.utils import scanner_state_reporter
from ostorlab.utils import ip

console = cli_console.Console()

logger = logging.getLogger(__name__)

WAIT_CAPTURE_INTERVAL = 300


async def _start_periodic_persist_state(
    state_reporter: scanner_state_reporter.ScannerStateReporter,
):
    while True:
        await state_reporter.report()
        logger.debug("Reporting the scanner state.")
        await asyncio.sleep(WAIT_CAPTURE_INTERVAL)


@rootcli.command()
@click.option("--scanner-id", help="The scanner identifier.", required=True)
@click.option("--daemon/--no-daemon", help="Run in daemon mode.", default=True)
@click.option(
    "--parallel",
    help="Number of scans to run in parallel.",
    default=1,
    type=click.IntRange(1, None),
)
@click.pass_context
def scanner(
    ctx: click.core.Context, daemon: bool, scanner_id: str, parallel: str
) -> None:
    """Ostorlab scanner enables running custom instances of scanners.
    Scanner communicates with NATs to receive start scan messages.\n
    """
    if sys.platform != "linux":
        console.error("ostorlab scanner sub-command is only supported on Unix systems.")
        raise click.exceptions.Exit(2)

    api_key = config_manager.ConfigurationManager().api_key or ctx.obj.get("api_key")

    # The import is done for Windows compatibility.
    import daemon as dm  # pylint: disable=import-outside-toplevel

    state_reporter = scanner_state_reporter.ScannerStateReporter(
        scanner_id=scanner_id, hostname=socket.gethostname(), ip=ip.get_ip()
    )
    nb_parallel_scans = int(parallel)
    processes = []

    for _ in range(nb_parallel_scans):
        process = multiprocessing.Process(
            target=start_scanner,
            args=(
                api_key,
                scanner_id,
                state_reporter,
            ),
        )
        process.start()
        processes.append(process)

    if daemon is True:
        with dm.DaemonContext():
            for process in processes:
                process.join()
    else:
        for process in processes:
            process.join()


def start_scanner(
    api_key: Optional[str],
    scanner_id: str,
    state_reporter: scanner_state_reporter.ScannerStateReporter,
) -> None:
    """Run subscription to nats in event loop.

    Args:
        api_key: The api key to login to the platform.
        scanner_id: The id of the scanner.
        state_reporter: instance responsible for reporting the scanner state.
    """
    if api_key is None:
        logger.error("No api key provided.")
    loop = asyncio.new_event_loop()
    loop.create_task(_start_periodic_persist_state(state_reporter=state_reporter))
    loop.run_until_complete(
        scan_handler.subscribe_nats(
            api_key=api_key,
            scanner_id=scanner_id,
            state_reporter=state_reporter,
        )
    )
    try:
        logger.debug("Closing loop.")
        loop.run_forever()
    finally:
        logger.debug("closing loop.")
        loop.close()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ostorlab/agent/message/proto/v3/fingerprint/file/android/metadata/metadata.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nPostorlab/agent/message/proto/v3/fingerprint/file/android/metadata/metadata.proto\x12\x41ostorlab.agent.message.proto.v3.fingerprint.file.android.metadata\">\n\x0cIntentFilter\x12\x0e\n\x06\x61\x63tion\x18\x01 \x01(\t\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\t\"O\n\nPermission\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x19\n\x11protection_levels\x18\x02 \x03(\t\x12\x18\n\x10permission_group\x18\x03 \x01(\t\"\x84\x02\n\x08\x41\x63tivity\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\npermission\x18\x02 \x01(\t\x12\x0f\n\x07process\x18\x03 \x01(\t\x12\x0f\n\x07\x65nabled\x18\x04 \x01(\x08\x12\x10\n\x08\x65xported\x18\x05 \x01(\x08\x12\x19\n\x11\x64irect_boot_aware\x18\x06 \x01(\x08\x12\x1e\n\x16\x61llow_task_reparenting\x18\x07 \x01(\x08\x12g\n\x0eintent_filters\x18\x08 \x03(\x0b\x32O.ostorlab.agent.message.proto.v3.fingerprint.file.android.metadata.IntentFilter\"\xfe\x01\n\x07Service\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\npermission\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\x0f\n\x07process\x18\x04 \x01(\t\x12\x0f\n\x07\x65nabled\x18\x05 \x01(\x08\x12\x10\n\x08\x65xported\x18\x06 \x01(\x08\x12\x1f\n\x17\x66oreground_service_type\x18\x07 \x01(\t\x12g\n\x0eintent_filters\x18\x08 \x03(\x0b\x32O.ostorlab.agent.message.proto.v3.fingerprint.file.android.metadata.IntentFilter\"\xf0\x01\n\x08Provider\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\npermission\x18\x02 \x01(\t\x12\x0f\n\x07process\x18\x03 \x01(\t\x12\x0f\n\x07\x65nabled\x18\x04 \x01(\x08\x12\x10\n\x08\x65xported\x18\x05 \x01(\x08\x12\x10\n\x08syncable\x18\x06 \x01(\x08\x12\x13\n\x0b\x61uthorities\x18\x07 \x03(\t\x12g\n\x0eintent_filters\x18\x08 \x03(\x0b\x32O.ostorlab.agent.message.proto.v3.fingerprint.file.android.metadata.IntentFilter\"\xe4\x01\n\x08Receiver\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\npermission\x18\x02 \x01(\t\x12\x0f\n\x07process\x18\x03 \x01(\t\x12\x0f\n\x07\x65nabled\x18\x04 \x01(\x08\x12\x10\n\x08\x65xported\x18\x05 \x01(\x08\x12\x19\n\x11\x64irect_boot_aware\x18\x06 \x01(\x08\x12g\n\x0eintent_filters\x18\x07 \x03(\x0b\x32O.ostorlab.agent.message.proto.v3.fingerprint.file.android.metadata.IntentFilter\"\x90\x04\n\x07Message\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x14\n\x0cpackage_name\x18\x02 \x01(\t\x12\x62\n\x0bpermissions\x18\x03 \x03(\x0b\x32M.ostorlab.agent.message.proto.v3.fingerprint.file.android.metadata.Permission\x12_\n\nactivities\x18\x04 \x03(\x0b\x32K.ostorlab.agent.message.proto.v3.fingerprint.file.android.metadata.Activity\x12\\\n\x08services\x18\x05 \x03(\x0b\x32J.ostorlab.agent.message.proto.v3.fingerprint.file.android.metadata.Service\x12^\n\tproviders\x18\x06 \x03(\x0b\x32K.ostorlab.agent.message.proto.v3.fingerprint.file.android.metadata.Provider\x12^\n\treceivers\x18\x07 \x03(\x0b\x32K.ostorlab.agent.message.proto.v3.fingerprint.file.android.metadata.Receiver')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ostorlab.agent.message.proto.v3.fingerprint.file.android.metadata.metadata_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _INTENTFILTER._serialized_start=151
  _INTENTFILTER._serialized_end=213
  _PERMISSION._serialized_start=215
  _PERMISSION._serialized_end=294
  _ACTIVITY._serialized_start=297
  _ACTIVITY._serialized_end=557
  _SERVICE._serialized_start=560
  _SERVICE._serialized_end=814
  _PROVIDER._serialized_start=817
  _PROVIDER._serialized_end=1057
  _RECEIVER._serialized_start=1060
  _RECEIVER._serialized_end=1288
  _MESSAGE._serialized_start=1291
  _MESSAGE._serialized_end=1819
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ostorlab/agent/message/proto/v3/fingerprint/file/ios/backend/backend.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nJostorlab/agent/message/proto/v3/fingerprint/file/ios/backend/backend.proto\x12<ostorlab.agent.message.proto.v3.fingerprint.file.ios.backend\"\xe5\x01\n\x08Location\x12\x11\n\tcontinent\x18\x01 \x01(\t\x12\x16\n\x0e\x63ontinent_code\x18\x02 \x01(\t\x12\x0f\n\x07\x63ountry\x18\x03 \x01(\t\x12\x14\n\x0c\x63ountry_code\x18\x04 \x01(\t\x12\x0e\n\x06region\x18\x05 \x01(\t\x12\x13\n\x0bregion_name\x18\x06 \x01(\t\x12\x0c\n\x04\x63ity\x18\x07 \x01(\t\x12\x0b\n\x03zip\x18\x08 \x01(\t\x12\x10\n\x08latitude\x18\t \x01(\x02\x12\x11\n\tlongitude\x18\n \x01(\x02\x12\x10\n\x08timezone\x18\x0b \x01(\t\x12\x10\n\x08\x64istrict\x18\x0c \x01(\t\"\x86\x01\n\x02Ip\x12\x12\n\nip_address\x18\x01 \x02(\t\x12\x12\n\x07version\x18\x02 \x01(\x05:\x01\x34\x12X\n\x08location\x18\x03 \x01(\x0b\x32\x46.ostorlab.agent.message.proto.v3.fingerprint.file.ios.backend.Location\"\x95\x01\n\x07Message\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x11\n\tbundle_id\x18\x02 \x02(\t\x12\x0c\n\x04host\x18\x03 \x02(\t\x12\x0c\n\x04port\x18\x04 \x01(\x05\x12M\n\x03ips\x18\x05 \x03(\x0b\x32@.ostorlab.agent.message.proto.v3.fingerprint.file.ios.backend.Ip')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ostorlab.agent.message.proto.v3.fingerprint.file.ios.backend.backend_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _LOCATION._serialized_start=141
  _LOCATION._serialized_end=370
  _IP._serialized_start=373
  _IP._serialized_end=507
  _MESSAGE._serialized_start=510
  _MESSAGE._serialized_end=659
# @@protoc_insertion_point(module_scope)



"""Module for the root command: scan."""
from ostorlab.cli.scan.scan import scan
from ostorlab.cli.scan.run import run
from ostorlab.cli.scan.stop import stop
from ostorlab.cli.scan.list import list as scan_list



"""Module for the command list inside the group scan.
This module takes care of listing all the remote or local scans.
Example of usage:
    - ostorlab scan list --source=source."""

import click

from ostorlab.cli import console as cli_console
from ostorlab.cli.scan import scan
from ostorlab.utils import styles

console = cli_console.Console()


@scan.command(name="list")
@click.option(
    "--page", "-p", help="Page number of scans you would like to see.", default=1
)
@click.option("--elements", "-e", help="Number of scans to show per page.", default=10)
@click.pass_context
def list_scans(ctx: click.core.Context, page: int, elements: int) -> None:
    """List all your scans.\n
    Usage:\n
        - ostorlab scan --runtime=runtime list
    """
    runtime_instance = ctx.obj["runtime"]
    with console.status("Fetching scans"):
        scans = runtime_instance.list(page=page, number_elements=elements)
        if scans is not None:
            console.success("Scans listed successfully.")
            columns = {
                "Id": "id",
                "Asset": "asset",
                "Created Time": "created_time",
                "Progress": "progress",
            }
            title = f"Showing {len(scans)} Scans"

            data = [
                {
                    "id": str(s.id),
                    "asset": s.asset,
                    "created_time": str(s.created_time),
                    "progress": styles.style_progress(s.progress),
                }
                for s in scans
            ]

            console.table(columns=columns, data=data, title=title)
        else:
            console.error("Error fetching scan list.")



"""Scan module that handles running a scan using a list of agent keys and a target asset."""

from typing import Optional

import click

from ostorlab.cli.rootcli import rootcli
from ostorlab.runtimes import registry
from ostorlab.cli import input_validators


@rootcli.group()
@click.option(
    "--runtime",
    type=click.Choice(["local", "litelocal", "cloud", "hybrid"]),
    help="""Runtime is in charge of preparing the environment to trigger a scan.\n
                    Specify which runtime to use: \n
                    local: on you local machine\n
                    litelocal: stripped down local runtime\n
                    cloud: on Ostorlab cloud, (requires login)\n
                    hybrid: running partially on Ostorlab cloud and partially on the local machine\n
                   """,
    default="local",
    required=True,
)
@click.option(
    "--bus-url",
    help="Bus URL, this flag is restricted to the lite local runtime.",
    required=False,
    hidden=True,
)
@click.option(
    "--bus-vhost",
    help="Bus vhost, this flag is restricted to the lite local runtime.",
    required=False,
    hidden=True,
)
@click.option(
    "--bus-management-url",
    help="Bus management URL, this flag is restricted to the lite local runtime.",
    required=False,
    hidden=True,
)
@click.option(
    "--bus-exchange-topic",
    help="Bus exchange topic, this flag is restricted to the lite local runtime.",
    required=False,
    hidden=True,
)
@click.option(
    "--network",
    help="Docker network to attach service and agents to.",
    required=False,
    hidden=True,
)
@click.option(
    "--redis-url",
    help="Redis URL, this flag is restricted to the lite local runtime.",
    required=False,
    hidden=True,
)
@click.option(
    "--scan-id",
    help="Scan id, this flag is restricted to the lite local runtime..",
    required=False,
    hidden=True,
)
@click.option("--tracing/--no-tracing", help="Enable tracing mode", default=False)
@click.option(
    "--tracing-collector-url",
    help="Tracing Collector URL, this flag is restricted to the lite local runtime.",
    required=False,
    hidden=True,
)
@click.option(
    "--mq-exposed-ports",
    help="Ports to expose on RabbitMQ service_port1:host_port1,service_port2:host_port2",
    required=False,
    hidden=True,
    callback=input_validators.validate_port_binding_input,
)
@click.pass_context
def scan(
    ctx: click.core.Context,
    runtime: str,
    bus_url: Optional[str] = None,
    bus_vhost: Optional[str] = None,
    bus_management_url: Optional[str] = None,
    bus_exchange_topic: Optional[str] = None,
    scan_id: Optional[str] = None,
    network: Optional[str] = None,
    redis_url: Optional[str] = None,
    tracing: bool = False,
    tracing_collector_url: Optional[str] = None,
    mq_exposed_ports: Optional[str] = None,
) -> None:
    """Use scan [subcommand] to list, start or stop a scan.\n
    Examples:\n
        - To show list of scans:\n
        ostorlab scan list\n
        - To stop a scan:\n
        ostorlab scan stop <scan-id>\n
    """
    try:
        if mq_exposed_ports is not None:
            exposed_ports_list = mq_exposed_ports.split(",")
            mq_exposed_ports = {
                int(port.split(":")[0]): int(port.split(":")[1])
                for port in exposed_ports_list
            }
        runtime_instance = registry.select_runtime(
            runtime,
            scan_id=scan_id,
            bus_url=bus_url,
            bus_vhost=bus_vhost,
            bus_management_url=bus_management_url,
            bus_exchange_topic=bus_exchange_topic,
            network=network,
            redis_url=redis_url,
            tracing=tracing,
            tracing_collector_url=tracing_collector_url,
            mq_exposed_ports=mq_exposed_ports,
            gcp_logging_credential=ctx.obj.get("gcp_logging_credential"),
        )
        ctx.obj["runtime"] = runtime_instance
    except registry.RuntimeNotFoundError as e:
        raise click.ClickException(
            f"The selected runtime {runtime} is not supported."
        ) from e
    except ValueError as e:
        raise click.ClickException(f"Error initializing runtime {runtime}.") from e



"""Check if requirements for running docker are satisfied."""

import docker
import platform
import sys
from docker import errors

_SUPPORTED_ARCH_TYPES = ["x86_64", "AMD64"]
# The architecture is checked with a return value that's based on the kernel implementation of the uname(2)
# system call. So it might be necesarry to handle the same arch with various strings e.g. linux returns x86_64
# or AMD64 on windows.


def is_docker_installed() -> bool:
    """Checks if docker is installed

    Returns:
        True if docker is installed, else False
    """
    try:
        _ = docker.from_env()
    except errors.DockerException as e:
        if "ConnectionRefusedError" in str(e):
            return False
    return True


def is_sys_arch_supported() -> bool:
    """Checks if the systems cpu architecture is supported

    Returns:
        True if the architecture is supported, else False
    """
    if sys.platform == "darwin" and "ARM" in platform.version():
        # On mac os, uname returns x86 even on arm64 if the process calling it is running via rosetta. We parse for ARM
        # in platform.version() to determine the arch on mac os
        return False
    else:
        if platform.machine() not in _SUPPORTED_ARCH_TYPES:
            return False
    return True


def is_user_permitted() -> bool:
    """Check if the user got permissions to run docker.

    Returns:
        True if user has permission to run docker, else False
    """
    try:
        _ = docker.from_env()
    except errors.DockerException as e:
        if "PermissionError" in str(e):
            return False
    return True


def is_docker_working() -> bool:
    """Last hope check to see if docker works without being able to give an intelligible recommendation.

    Returns:
        True if user has permission to run docker, else False
    """
    if sys.platform == "win32":
        import pywintypes  # pylint: disable=import-outside-toplevel

        try:
            client = docker.from_env()
            client.ping()
        except errors.DockerException:
            return False
        except pywintypes.error:
            return False
    else:
        try:
            client = docker.from_env()
            client.ping()
        except errors.DockerException:
            return False
    return True


def is_swarm_initialized() -> bool:
    """Checks if docker swarm is initialized.

    Returns:
        True if docker swarm is initialized, else False
    """
    if is_user_permitted():
        docker_client = docker.from_env()
        if docker_client.swarm.id is None:
            return False
        else:
            return True
    else:
        return False


def init_swarm() -> None:
    """Initialize docker swarm"""
    if is_user_permitted():
        docker_client = docker.from_env()
        docker_client.swarm.init()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ostorlab/scanner/proto/scan/_location/startAgentScan.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ostorlab.scanner.proto.assets import ip_pb2 as ostorlab_dot_scanner_dot_proto_dot_assets_dot_ip__pb2
from ostorlab.scanner.proto.assets import v4_pb2 as ostorlab_dot_scanner_dot_proto_dot_assets_dot_v4__pb2
from ostorlab.scanner.proto.assets import v6_pb2 as ostorlab_dot_scanner_dot_proto_dot_assets_dot_v6__pb2
from ostorlab.scanner.proto.assets import domain_name_pb2 as ostorlab_dot_scanner_dot_proto_dot_assets_dot_domain__name__pb2
from ostorlab.scanner.proto.assets import file_pb2 as ostorlab_dot_scanner_dot_proto_dot_assets_dot_file__pb2
from ostorlab.scanner.proto.assets import aab_pb2 as ostorlab_dot_scanner_dot_proto_dot_assets_dot_aab__pb2
from ostorlab.scanner.proto.assets import apk_pb2 as ostorlab_dot_scanner_dot_proto_dot_assets_dot_apk__pb2
from ostorlab.scanner.proto.assets import ipa_pb2 as ostorlab_dot_scanner_dot_proto_dot_assets_dot_ipa__pb2
from ostorlab.scanner.proto.assets import source_pb2 as ostorlab_dot_scanner_dot_proto_dot_assets_dot_source__pb2
from ostorlab.scanner.proto.assets import agent_pb2 as ostorlab_dot_scanner_dot_proto_dot_assets_dot_agent__pb2
from ostorlab.scanner.proto.assets import network_pb2 as ostorlab_dot_scanner_dot_proto_dot_assets_dot_network__pb2
from ostorlab.scanner.proto.assets import link_pb2 as ostorlab_dot_scanner_dot_proto_dot_assets_dot_link__pb2
from ostorlab.scanner.proto.assets import android_store_pb2 as ostorlab_dot_scanner_dot_proto_dot_assets_dot_android__store__pb2
from ostorlab.scanner.proto.assets import ios_store_pb2 as ostorlab_dot_scanner_dot_proto_dot_assets_dot_ios__store__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n:ostorlab/scanner/proto/scan/_location/startAgentScan.proto\x12\x34ostorlab.scanner.proto.scan._location.startAgentScan\x1a&ostorlab/scanner/proto/assets/ip.proto\x1a&ostorlab/scanner/proto/assets/v4.proto\x1a&ostorlab/scanner/proto/assets/v6.proto\x1a/ostorlab/scanner/proto/assets/domain_name.proto\x1a(ostorlab/scanner/proto/assets/file.proto\x1a\'ostorlab/scanner/proto/assets/aab.proto\x1a\'ostorlab/scanner/proto/assets/apk.proto\x1a\'ostorlab/scanner/proto/assets/ipa.proto\x1a*ostorlab/scanner/proto/assets/source.proto\x1a)ostorlab/scanner/proto/assets/agent.proto\x1a+ostorlab/scanner/proto/assets/network.proto\x1a(ostorlab/scanner/proto/assets/link.proto\x1a\x31ostorlab/scanner/proto/assets/android_store.proto\x1a-ostorlab/scanner/proto/assets/ios_store.proto\"0\n\x03\x41rg\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x0c\n\x04type\x18\x02 \x02(\t\x12\r\n\x05value\x18\x03 \x01(\x0c\"\xcf\x01\n\x05\x41gent\x12\x0b\n\x03key\x18\x01 \x02(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x10\n\x08replicas\x18\x03 \x01(\x05\x12G\n\x04\x61rgs\x18\x04 \x03(\x0b\x32\x39.ostorlab.scanner.proto.scan._location.startAgentScan.Arg\x12\x0c\n\x04\x63\x61ps\x18\x05 \x03(\t\x12\x1f\n\x17\x63yclic_processing_limit\x18\x06 \x01(\r\x12\x1e\n\x16\x64\x65pth_processing_limit\x18\x07 \x01(\r\"C\n\x05Links\x12:\n\x05links\x18\x01 \x03(\x0b\x32+.ostorlab.scanner.proto.assets.link.Message\"\xa7\x08\n\x07Message\x12\x19\n\x11reference_scan_id\x18\x01 \x02(\x05\x12K\n\x06\x61gents\x18\x02 \x03(\x0b\x32;.ostorlab.scanner.proto.scan._location.startAgentScan.Agent\x12\x0b\n\x03key\x18\x11 \x01(\t\x12\x37\n\x02ip\x18\x03 \x01(\x0b\x32).ostorlab.scanner.proto.assets.ip.MessageH\x00\x12\x39\n\x04ipv4\x18\x04 \x01(\x0b\x32).ostorlab.scanner.proto.assets.v4.MessageH\x00\x12\x39\n\x04ipv6\x18\x05 \x01(\x0b\x32).ostorlab.scanner.proto.assets.v6.MessageH\x00\x12;\n\x04\x66ile\x18\x06 \x01(\x0b\x32+.ostorlab.scanner.proto.assets.file.MessageH\x00\x12\x39\n\x03\x61\x61\x62\x18\x07 \x01(\x0b\x32*.ostorlab.scanner.proto.assets.aab.MessageH\x00\x12\x39\n\x03\x61pk\x18\x08 \x01(\x0b\x32*.ostorlab.scanner.proto.assets.apk.MessageH\x00\x12\x39\n\x03ipa\x18\t \x01(\x0b\x32*.ostorlab.scanner.proto.assets.ipa.MessageH\x00\x12?\n\x06source\x18\n \x01(\x0b\x32-.ostorlab.scanner.proto.assets.source.MessageH\x00\x12I\n\x0b\x64omain_name\x18\x0b \x01(\x0b\x32\x32.ostorlab.scanner.proto.assets.domain_name.MessageH\x00\x12=\n\x05\x61gent\x18\x0c \x01(\x0b\x32,.ostorlab.scanner.proto.assets.agent.MessageH\x00\x12\x41\n\x07network\x18\r \x01(\x0b\x32..ostorlab.scanner.proto.assets.network.MessageH\x00\x12L\n\x05links\x18\x0e \x01(\x0b\x32;.ostorlab.scanner.proto.scan._location.startAgentScan.LinksH\x00\x12M\n\randroid_store\x18\x0f \x01(\x0b\x32\x34.ostorlab.scanner.proto.assets.android_store.MessageH\x00\x12\x45\n\tios_store\x18\x10 \x01(\x0b\x32\x30.ostorlab.scanner.proto.assets.ios_store.MessageH\x00\x12\x0f\n\x07scan_id\x18\x12 \x01(\x05\x42\x07\n\x05\x61sset')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ostorlab.scanner.proto.scan._location.startAgentScan_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _ARG._serialized_start=722
  _ARG._serialized_end=770
  _AGENT._serialized_start=773
  _AGENT._serialized_end=980
  _LINKS._serialized_start=982
  _LINKS._serialized_end=1049
  _MESSAGE._serialized_start=1052
  _MESSAGE._serialized_end=2115
# @@protoc_insertion_point(module_scope)



"""Handles the creation of an API key."""

from ostorlab.apis import request
import datetime
from typing import Dict, Optional
import json
import logging

logger = logging.getLogger(__name__)


class CreateAPIKeyAPIRequest(request.APIRequest):
    """Handles the creation of an API key."""

    def __init__(self, expiry_date: Optional[datetime.datetime] = None):
        """Constructs all the necessary attributes for the object.

        Args:
           expiry_date: The date when the API key should expire. Defaults to None.
        """
        self._name = "Ostorlab CLI"
        self._expiry_date = expiry_date

    @property
    def query(self) -> Optional[str]:
        """Defines the query to generate an API key.

        Returns:
            The query to generate an API key
        """
        return """
         mutation CreateApiKey($name: String!, $expiryDate: DateTime) {
               createApiKey(name: $name, expiryDate: $expiryDate) {
                  apiKey {
                     secretKey
                     apiKey {
                        expiryDate
                        id
                     }
                  }
               }
            }
        """

    @property
    def data(self) -> Optional[Dict]:
        """Sets the query and variables to generate an API key.

        Returns:
              The API key, name, and expiry date.
        """
        data = {
            "query": self.query,
            "variables": json.dumps(
                {"name": self._name, "expiryDate": self._expiry_date}
            ),
        }
        return data



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ostorlab/agent/message/proto/v3/report/cve/cve.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n4ostorlab/agent/message/proto/v3/report/cve/cve.proto\x12*ostorlab.agent.message.proto.v3.report.cve\"\xc1\x02\n\x07Message\x12\x0e\n\x06\x63ve_id\x18\x01 \x01(\t\x12\x43\n\x07targets\x18\x02 \x03(\x0b\x32\x32.ostorlab.agent.message.proto.v3.report.cve.Target\x12\x0b\n\x03\x63we\x18\x03 \x01(\x05\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\x12I\n\nreferences\x18\x05 \x03(\x0b\x32\x35.ostorlab.agent.message.proto.v3.report.cve.Reference\x12\x45\n\x0b\x63vss_scores\x18\x06 \x01(\x0b\x32\x30.ostorlab.agent.message.proto.v3.report.cve.CVSS\x12\x16\n\x0epublished_date\x18\x07 \x01(\x04\x12\x15\n\rmodified_date\x18\x08 \x01(\x04\"\xc8\x01\n\x04\x43VSS\x12?\n\x03v31\x18\x01 \x01(\x0b\x32\x32.ostorlab.agent.message.proto.v3.report.cve.CVSSv3\x12?\n\x03v30\x18\x02 \x01(\x0b\x32\x32.ostorlab.agent.message.proto.v3.report.cve.CVSSv3\x12>\n\x02v2\x18\x03 \x01(\x0b\x32\x32.ostorlab.agent.message.proto.v3.report.cve.CVSSv2\"\xdb\x05\n\x06\x43VSSv2\x12\x0f\n\x07version\x18\x01 \x01(\t\x12\x15\n\rvector_string\x18\x02 \x01(\t\x12\x1e\n\x16\x63onfidentiality_impact\x18\x03 \x01(\t\x12\x18\n\x10integrity_impact\x18\x04 \x01(\t\x12\x1b\n\x13\x61vailability_impact\x18\x05 \x01(\t\x12\x12\n\nbase_score\x18\x06 \x01(\x02\x12\x19\n\x11remediation_level\x18\x07 \x01(\t\x12\x19\n\x11report_confidence\x18\x08 \x01(\t\x12\x16\n\x0etemporal_score\x18\t \x01(\x02\x12#\n\x1b\x63onfidentiality_requirement\x18\n \x01(\t\x12\x1d\n\x15integrity_requirement\x18\x0b \x01(\t\x12 \n\x18\x61vailability_requirement\x18\x0c \x01(\t\x12\x1b\n\x13\x65nvironmental_score\x18\r \x01(\x02\x12\x15\n\raccess_vector\x18\x0e \x01(\t\x12\x19\n\x11\x61\x63\x63\x65ss_complexity\x18\x0f \x01(\t\x12\x16\n\x0e\x61uthentication\x18\x10 \x01(\t\x12\x16\n\x0e\x65xploitability\x18\x11 \x01(\t\x12#\n\x1b\x63ollateral_damage_potential\x18\x12 \x01(\t\x12\x1b\n\x13target_distribution\x18\x13 \x01(\t\x12\x14\n\x0cimpact_score\x18\x14 \x01(\x02\x12\x1c\n\x14\x65xploitability_score\x18\x15 \x01(\x02\x12\x15\n\rac_insuf_info\x18\x16 \x01(\x08\x12\x1c\n\x14obtain_all_privilege\x18\x17 \x01(\x08\x12\x1d\n\x15obtain_user_privilege\x18\x18 \x01(\x08\x12\x1e\n\x16obtain_other_privilege\x18\x19 \x01(\x08\x12!\n\x19user_interaction_required\x18\x1a \x01(\x08\"\xa0\x07\n\x06\x43VSSv3\x12\x0f\n\x07version\x18\x01 \x01(\t\x12\x15\n\rvector_string\x18\x02 \x01(\t\x12\x1e\n\x16\x63onfidentiality_impact\x18\x03 \x01(\t\x12\x18\n\x10integrity_impact\x18\x04 \x01(\t\x12\x1b\n\x13\x61vailability_impact\x18\x05 \x01(\t\x12\x12\n\nbase_score\x18\x06 \x01(\x02\x12\x19\n\x11remediation_level\x18\x07 \x01(\t\x12\x19\n\x11report_confidence\x18\x08 \x01(\t\x12\x16\n\x0etemporal_score\x18\t \x01(\x02\x12#\n\x1b\x63onfidentiality_requirement\x18\n \x01(\t\x12\x1d\n\x15integrity_requirement\x18\x0b \x01(\t\x12 \n\x18\x61vailability_requirement\x18\x0c \x01(\t\x12\x1b\n\x13\x65nvironmental_score\x18\r \x01(\x02\x12\x15\n\rattack_vector\x18\x0e \x01(\t\x12\x19\n\x11\x61ttack_complexity\x18\x0f \x01(\t\x12\x1b\n\x13privileges_required\x18\x10 \x01(\t\x12\x18\n\x10user_interaction\x18\x11 \x01(\t\x12\r\n\x05scope\x18\x12 \x01(\t\x12\x15\n\rbase_severity\x18\x13 \x01(\t\x12\x1d\n\x15\x65xploit_code_maturity\x18\x14 \x01(\t\x12\x19\n\x11temporal_severity\x18\x15 \x01(\t\x12\x1e\n\x16modified_attack_vector\x18\x16 \x01(\t\x12\"\n\x1amodified_attack_complexity\x18\x17 \x01(\t\x12$\n\x1cmodified_privileges_required\x18\x18 \x01(\t\x12!\n\x19modified_user_interaction\x18\x19 \x01(\t\x12\x16\n\x0emodified_scope\x18\x1a \x01(\t\x12\'\n\x1fmodified_confidentiality_impact\x18\x1b \x01(\t\x12!\n\x19modified_integrity_impact\x18\x1c \x01(\t\x12$\n\x1cmodified_availability_impact\x18\x1d \x01(\t\x12\x1e\n\x16\x65nvironmental_severity\x18\x1e \x01(\t\x12\x14\n\x0cimpact_score\x18\x1f \x01(\x02\x12\x1c\n\x14\x65xploitability_score\x18  \x01(\x02\"6\n\tReference\x12\x0b\n\x03url\x18\x01 \x01(\t\x12\x0e\n\x06source\x18\x02 \x01(\t\x12\x0c\n\x04tags\x18\x03 \x03(\t\"V\n\x0cVersionMatch\x12\r\n\x05start\x18\x01 \x01(\t\x12\x15\n\rstart_include\x18\x02 \x01(\x08\x12\x0b\n\x03\x65nd\x18\x03 \x01(\t\x12\x13\n\x0b\x65nd_include\x18\x04 \x01(\x08\"\x86\x02\n\x08\x43peMatch\x12\x0c\n\x04part\x18\x01 \x01(\t\x12\x0e\n\x06vendor\x18\x02 \x01(\t\x12\x0f\n\x07product\x18\x03 \x01(\t\x12I\n\x07version\x18\x04 \x01(\x0b\x32\x38.ostorlab.agent.message.proto.v3.report.cve.VersionMatch\x12\x0e\n\x06update\x18\x05 \x01(\t\x12\x0f\n\x07\x65\x64ition\x18\x06 \x01(\t\x12\x10\n\x08language\x18\x07 \x01(\t\x12\x12\n\nsw_edition\x18\x08 \x01(\t\x12\n\n\x02sw\x18\t \x01(\t\x12\n\n\x02hw\x18\n \x01(\t\x12\r\n\x05other\x18\x0b \x01(\t\x12\x12\n\nvulnerable\x18\x0c \x01(\x08\"S\n\x06Target\x12I\n\x0b\x63pe_matches\x18\x01 \x03(\x0b\x32\x34.ostorlab.agent.message.proto.v3.report.cve.CpeMatch')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ostorlab.agent.message.proto.v3.report.cve.cve_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _MESSAGE._serialized_start=101
  _MESSAGE._serialized_end=422
  _CVSS._serialized_start=425
  _CVSS._serialized_end=625
  _CVSSV2._serialized_start=628
  _CVSSV2._serialized_end=1359
  _CVSSV3._serialized_start=1362
  _CVSSV3._serialized_end=2290
  _REFERENCE._serialized_start=2292
  _REFERENCE._serialized_end=2346
  _VERSIONMATCH._serialized_start=2348
  _VERSIONMATCH._serialized_end=2434
  _CPEMATCH._serialized_start=2437
  _CPEMATCH._serialized_end=2699
  _TARGET._serialized_start=2701
  _TARGET._serialized_end=2784
# @@protoc_insertion_point(module_scope)



"""Module for the root command: agent."""
from ostorlab.cli.agent.agent import agent
from ostorlab.cli.agent import build
from ostorlab.cli.agent import install
from ostorlab.cli.agent import healthcheck
from ostorlab.cli.agent import delete
from ostorlab.cli.agent import list as list_cli
from ostorlab.cli.agent import search as search_cli
